/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/bdhk.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "fourie.h"
#include "modefl.h"
#include "consts.h"
#include "uiod.h"
#include "anglst.h"
#include "errfac.h"
#include "errpar.h"
#include "pick.h"
#include "pseudo.h"
#include <cstdio>
#include <iostream>
#include "infile.h"
#include "anglst.h"
#include <iomanip>
#include <fstream>
#include "second.h"
#include "wray.h"
#include "raysel.h"
#include "bdhk.h"
using namespace std;

INTEGER bdhk_class::Init()
{

	flag_kount = TRUE;

	if (RaySel.useray)
		kount = GeoPar.prjnum * GeoPar.usrays;
	else
		kount = GeoPar.prjnum * GeoPar.snrays;
	//finishNow=FALSE;
	arrayRays = new struct NPNR[kount];
	orderRays(arrayRays, kount);

	//Lastrecon = new REAL[GeoPar.area]; // the array to keep the recon of the least TV seen so far
	//total_time=0;
	return 0;
}

INTEGER bdhk_class::Reset()
{
	delete[] arrayRays;
	//delete [] Lastrecon;
	return 0;
}

BOOLEAN bdhk_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{

	INTEGER count = 0;

	while (flag_kount)
	{

		REAL resX = 0;
		REAL* v = new REAL[GeoPar.area];
		REAL* d = new REAL[GeoPar.area];

		if (iter == 1 && count == 0)
		{
			BOOLEAN eol;
			REAL word;

			//checking for BETA
			word = InFile.getnum(TRUE, &eol);
			if (eol)
			{
				fprintf(output,
						"\n          **** error - you must specify 1 parameter list ***");
				return TRUE;
			}
			BETA = word;
			if (BETA <= Consts.zero)
			{
				fprintf(output,
						"\n          **** error - BETA = %10.6f must be greater than %12.4e***",
						word, Consts.zero);
				return TRUE;
			}
			else
				fprintf(output, "\n BETA       = %10.6f", word);

			//checking for kount
			word = InFile.getnum(FALSE, &eol);
			if (eol)
			{
				fprintf(output, "\n          **** error - parameter list***");
				return TRUE;
			}

			if (word <= 0)
			{
				if (RaySel.useray)
					KOUNT = GeoPar.prjnum * GeoPar.usrays;
				else
					KOUNT = GeoPar.prjnum * GeoPar.snrays;
				fprintf(output,
						"\n        KOUNT remained in default, KOUNT = %d                                       ***",
						KOUNT);

			}
			else
			{
				KOUNT = word;
				fprintf(output, "\n KOUNT        = %10.6f", word);
			}

		}  //end if (iter==1 && count==0)

		resX = calculateResidual(recon, list, weight, arrayRays, kount);

		BETA *= 2;

		REAL* grad = new REAL[GeoPar.area];
		REAL* reconBU = new REAL[GeoPar.area];

		BOOLEAN flag = calculateGradientTV(grad, recon);

		if (flag == FALSE)
		{
			//initialiaze the d vector to all zero
			for (int iter1 = 0; iter1 < GeoPar.area; iter1++)
				d[iter1] = 0;
		}
		else
		{
			//have d vector equal to g/||g||
			REAL norm_grad = calcNorm(grad);

			if (trace == 1)
				cout << endl << "norm_grad=" << norm_grad << "    " << endl
						<< endl;

			for (int iter1 = 0; iter1 < GeoPar.area; iter1++)
				d[iter1] = grad[iter1] / norm_grad;

		}

		REAL resAv;
		REAL TVx = calcTV(recon);

		if (trace == 1)
			cout << endl << endl << "TVx=" << TVx << "    ";

		REAL TVv;
		BOOLEAN logic = TRUE;
		do
		{
			if (trace == 1)
				cout << endl << "entered DO-WHILE" << endl;

			resAv = resX + 1;
			BETA /= 2;

			if (trace == 1)
				cout << endl << "BETA=" << BETA << "    " << endl;

			for (int iter1 = 0; iter1 < GeoPar.area; iter1++)
				v[iter1] = recon[iter1] - BETA * d[iter1];

			TVv = calcTV(v);

			if (trace == 1)
				cout << "TVv=" << TVv << "    " << endl;
//*************************************************************************
//CHECKING THE CONDITION:
//~~~~~~~~~~~~~~~~~~~~~~~
			if (TVv <= TVx)
			{

				//Backing Up recon array as v is going to be passed to MyART
				for (int iter1 = 0; iter1 < GeoPar.area; iter1++)
				{
					reconBU[iter1] = recon[iter1];
					recon[iter1] = v[iter1];
				}  //now recon contains the vector v and will be passed to MyART

				if (trace == 1)
					cout << endl << "resX=" << resX << "   ";

				MyART(recon, list, weight, arrayRays, kount);
				count += kount;

				resAv = calculateResidual(recon, list, weight, arrayRays,
						kount);

				//if (trace == 1)
				//cout<<endl<<"resX="<<resX<<"   ";
				if (trace == 1)
					cout << "resAv=" << resAv << endl << endl;

				if (resAv >= resX) //only if the condition in the while loop kept as TRUE, we let recon to go back to its previous state. Otherwise stays as X_k+1
				{
					for (int iter1 = 0; iter1 < GeoPar.area; iter1++)
						recon[iter1] = reconBU[iter1];
				}
				else
					logic = FALSE;

			} // end if (TVv <= TVx)
//**************************************************************************
		} while (logic);

		if (count >= KOUNT)
			flag_kount = FALSE;

//deallocation of memory:
		delete[] d;
		delete[] v;
		delete[] grad;
		delete[] reconBU;

		if (trace == 1)
			cout << "iter " << iter << " ENDED with BETA=" << BETA << endl
					<< endl;

	} //end while (flag_kount)

//returning snark iteration
	flag_kount = TRUE;
	return FALSE;

} //end of snark iteration

