/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/blob_bpix.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 subroutine to perform a task similar to bwray() but pixels instead of the rays

 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "blob.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"

void Blob_class::bpix(INTEGER nv, INTEGER nh, INTEGER *list, REAL* weight,
		INTEGER *numb, REAL *snorm)
{
	INTEGER multp;
	INTEGER m, h, m1, h1;
	INTEGER mc, hc;
	INTEGER mf, hf, ml, hl;
	INTEGER M2, H2;
	INTEGER nelem2;
	INTEGER mx, hy;
	INTEGER ind, n;
	INTEGER m0, h0;

	REAL xc, yc;
	REAL dinv2 = (REAL) 2.0 / Blob.delta;
	REAL dinv2sq3 = dinv2 * Consts.isqrt3;

	REAL dist, distx, disty; // dist is the distance betw a blob center and a pixel center. (distx, disty) is the vector distance
	REAL dispx, dispy; // (dispx, dispy) is the displacement from the center of a blob to the center of the nearest pixel.

	multp = (INTEGER) floor(Blob.support / Blob.delta + 0.5) + 1;

	H2 = (INTEGER) (Blob.H / 2);
	M2 = (INTEGER) (Blob.M / 2);
	nelem2 = (INTEGER) (GeoPar.nelem / 2);

	REAL* footpx = new REAL[4 * multp + 1];
	REAL* footpy = new REAL[4 * multp + 1];

	for (m0 = -2 * multp; m0 <= 2 * multp; m0++)
	{
		footpx[m0 + 2 * multp] = (REAL) m0 * Blob.di2;
	}

	for (h0 = -2 * multp; h0 <= 2 * multp; h0++)
	{
		footpy[h0 + 2 * multp] = (REAL) h0 * Blob.di2sq3;
	}

// coord of the center of the pixel (nh, nv)
	xc = GeoPar.pixsiz * (nh - nelem2);
	yc = GeoPar.pixsiz * (nelem2 - nv);

	mx = (INTEGER) floor(dinv2 * xc + 0.5); // coord of the center of

	hy = (INTEGER) floor(dinv2sq3 * yc + 0.5);        // the nearest blob

	mc = mx;
	hc = hy;

	// displacement of the pixel footprint with resp to the nearest blob center
	dispx = di2 * mx - xc;
	dispy = di2sq3 * hy - yc;

	mf = MAX0(lmf, mc - 2 * multp);
	ml = MIN0(lml, mc + 2 * multp);

	hf = MAX0(lhf, hc - 2 * multp);
	hl = MIN0(lhl, hc + 2 * multp);

	(*numb) = 0;
	(*snorm) = 0.0;

	if (ml < lmf || mf > lml || hl < lhf || hf > lhl)
		return;
	for (h = hf; h <= hl; h++)
	{
		h0 = h - hc;
		disty = footpy[h0 + 2 * multp] + dispy; //the y in the x-y coord and the h in the m-h coord are in the same direction
		if (fabs(disty) > Blob.support)
			continue;

		for (m = mf; m <= ml; m++)
		{
			m0 = m - mc;
			distx = footpx[m0 + 2 * multp] + dispx;
			if (fabs(distx) > Blob.support)
				continue;

			if ((m + h) % 2 == 0)
			{
				m1 = m + M2;
				h1 = h + H2;
				ind = h1 * Blob.M + m1;

				dist = sqrt(distx * distx + disty * disty);
				if (dist < Blob.support)
				{
					n = (INTEGER) round(dist * sampiblob); // changed "rint" to "round". Lajos, Jan 25, 2005
					list[(*numb)] = ind;
					weight[(*numb)] = Blob.vtable[n];

					(*snorm) += (Blob.vtable[n] * Blob.vtable[n]);
					(*numb)++;
				}
			} //for m+h
		} // for m
	}  //for h

	delete[] footpx;  // bug 92 - Lajos - 03/02/2005
	delete[] footpy;  // bug 92 - Lajos - 03/02/2005
	return;
}

