/*
 **********************************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/doline.cpp $
 $LastChangedRevision: 79 $
 $Date: 2014-07-01 15:33:39 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 **********************************************************************

 POST-PROCESSING ROUTINE TO GENERATE LINE DISPLAYS OF
 RECONSTRUCTIONS.
 */

#include <cstdio>

#include "blkdta.h"
#include "uiod.h"

#include "lines.h"

//#include "int2str.h"
#include "recfile.h"
#include "infile.h"

#include "doline.h"

void doline()
{
	// was in post but it is realy local
	CHAR prjnam[81];
	CHAR phnnam[81];
	CHAR recnam[81];

	// was in geo but it is realy local
	INTEGER nelem;
	INTEGER area;

	REAL scale, denmin, denmax;
	INTEGER word, lineno, linumb[4], flag[51];
	REAL* Phantom;
	REAL* Recon;
	CHAR algn[5];
	unsigned INTEGER count, iter;
	BOOLEAN eol;
	BOOLEAN minfl, maxfl;

	INTEGER index;

	int i;

	static const INTEGER hscal = CHAR2INT('s', 'c', 'a', 'l');
	static const INTEGER hcolu = CHAR2INT('c', 'o', 'l', 'u');

	// modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 13, 2004
	static const INTEGER doline_codes[2] = { CHAR2INT('s', 'c', 'a', 'l'), CHAR2INT('c', 'o', 'l', 'u') };

	if (RecFile.Open("recfil") != 0)
	{
		fprintf(output, "\n **** unable to open recfil");
		fprintf(output, "\n **** LINE execution aborted\n");
		return;
	}

	if (RecFile.GetNelem(&nelem) != 0)
	{
		;
	}

	if (RecFile.GetProjName(prjnam) != 0)
	{
		;
	}

	area = nelem * nelem;
	//GeoPar.midpix = (GeoPar.nelem + 1) / 2;

	lineno = 0;
	minfl = FALSE;
	maxfl = FALSE;
	scale = 1.0;

	for (;;)
	{
		word = InFile.getwrd(FALSE, &eol, doline_codes, 2);

		if (eol)
		{
			fprintf(output, "\n **** no columns requested");
			fprintf(output, "\n **** LINE execution aborted\n");
			RecFile.Close();
			return;
		}

		if (word == hscal)
		{
			scale = InFile.getnum(FALSE, &eol);
			continue;
		};

		if (word == hcolu)
			break;
	}

	for (;;)
	{
		index = InFile.getint(FALSE, &eol);

		if (eol)
			break;

		if ((index >= 0) && (index < nelem))
		{

			if (lineno >= 4)
				break;
			linumb[lineno++] = index;
		}
		else
		{
			fprintf(output,
					"\n **** requested column %d is not in the range (0, %d)",
					index, nelem - 1);
			fprintf(output, "\n **** COLUMN %d ignored", index);
		}

	}

	if (lineno == 0)
	{
		fprintf(output, "\n **** no columns requested");
		fprintf(output, "\n **** LINE execution aborted\n");
		RecFile.Close();
		return;
	}

	InFile.listit(flag);

	Phantom = new REAL[area];
	Recon = new REAL[area];

	// load phantom
	if (RecFile.ReadPhan(phnnam, Phantom) != 0)
	{
		fprintf(output, "\n **** phantom not present");
		fprintf(output, "\n **** LINE execution aborted\n");
		RecFile.Close();
		return;
	}

	if (!minfl)
	{
		denmin = Phantom[0];
		for (i = 1; i < area; i++)
		{
			if (Phantom[i] < denmin)
				denmin = Phantom[i];
		}
	}
	if (!maxfl)
	{
		denmax = Phantom[0];
		for (i = 1; i < area; i++)
		{
			if (Phantom[i] > denmax)
				denmax = Phantom[i];
		}
	}

	// Load reconstructed images
	while (RecFile.ReadRec(recnam, algn, &count, &iter, Recon) == 0)
	{

		if (flag[iter - 1] != 0)
		{
			fprintf(output,
					"\n     lines display of reconstruction using %s iter %d\n",
					algn, count);
			fprintf(output, "\n     phantom name:    %s", phnnam);
			fprintf(output, "\n     projection data: %s", prjnam);
			fprintf(output, "\n     execution name:  %s", recnam);
			lines(Phantom, Recon, nelem, nelem, scale, lineno, linumb, output);
		}
	}

	delete[] Recon;  // bug 92 - Lajos - 03/02/2005
	delete[] Phantom;  // bug 92 - Lajos - 03/02/2005

	fprintf(output, "\n ");

	RecFile.Close();
	return;
}
