/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/experimenter.c $
 $LastChangedRevision: 177 $
 $Date: 2015-08-03 16:44:07 -0400 (Mon, 03 Aug 2015) $
 $Author: zye $
 ***********************************************************

 Previously part of SuperSNARK
 */

/* --------------------------- experimenter.c -----------------------------

 This is the main program. It reads input directly from the keyboard
 or from a redirected file.

 --------------------------------------------------------------------------- */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

#include "experimenter.h"
#include "errorc.h"
#include "supersnk.h"

#ifdef WIN32

#include <string.h>

int strncasecmp(char* str1, char* str2, int len)
{
	return strncmp(_strupr(str1), _strupr(str2), len);
}

#endif

void errorchk1(int nexp, int expflg, int nphan1, int nrun1);

int experimenter(int argc, char* argv[])
{
	char string[MAXLINESIZE], ensflname[MAXLINESIZE], projflname[MAXLINESIZE],
			recflname[MAXLINESIZE], anaflname[MAXLINESIZE], tmp[MAXLINESIZE];
	int nexp, expflg, nelem, navel, nphan[MAXEXP], nrun[MAXEXP];
	float pixsz;
	long seed;

	char progName[256];
	strcpy(progName, argv[0]);

	fprintf(stdout, "snark14Experimenter.s170426\n");

	if (argc > 1)
	{
		close(fileno(stdin));
		if (open(argv[2], O_RDONLY) < 0)
		{
			perror(argv[2]);
			exit(1);
		}
	}

	while (1)
	{
		// optional SEED line  and  ENSEMBLE line
		if (fgets(string, sizeof(string), stdin) == NULL)
		{
			fprintf(stdout, "input error: end of file reached\n");
			break;
		}
		while (isSkip(string))     // skip comment lines and blank lines
		{
			if (fgets(string, sizeof(string), stdin) == NULL)
			{
				fprintf(stdout, "input error: end of file reached\n");
				break;
			}
		}
		if (strncasecmp(string, "see", 3) == 0)
		{
			if (1 == sscanf(string, "%*s%ld", &seed))
			{
				if (seed < 0)
					time(&seed);
				srand48(seed);
			}
			else
			{
				errorc("Error reading supersnark input file at ", string);
			}
			// now read the ens line
			if (fgets(string, sizeof(string), stdin) == NULL)
			{
				fprintf(stdout, "input error: end of file reached\n");
				break;
			}
			while (isSkip(string))     // skip comment lines and blank lines
			{
				if (fgets(string, sizeof(string), stdin) == NULL)
				{
					fprintf(stdout, "input error: end of file reached\n");
					break;
				}
			}
			if (strncasecmp(string, "ens", 3) == 0)
			{
				sscanf(string, "%*s%s", ensflname);
			}
			else if (strncasecmp(string, "end", 3) == 0)
			{
				break;
			}
			else
			{
				errorc("Error reading supersnark input file at ", string);
				break;
			}

		}
		else if (strncasecmp(string, "ens", 3) == 0)
		{
			sscanf(string, "%*s%s", ensflname);
			// there is not seed line, so set seed to current time
			time(&seed);
			srand48(seed);
		}
		else if (strncasecmp(string, "end", 3) == 0)
		{
			break;
		}
		else
		{
			errorc("Error reading supersnark input file at ", string);
			break;
		}

		if (fgets(string, sizeof(string), stdin) == NULL)
		{
			fprintf(stdout, "input error: end of file reached\n");
			break;
		}
		while (isSkip(string))     // skip comment lines and blank lines
		{
			if (fgets(string, sizeof(string), stdin) == NULL)
			{
				fprintf(stdout, "input error: end of file reached\n");
				break;
			}
		}
		if (strncasecmp(string, "exp", 3) == 0)
		{

			sscanf(string, "%*s %d %d %d %f %d %d %d", &nexp, &expflg, &nelem,
					&pixsz, &navel, &nphan[0], &nrun[0]);
			errorchk1(nexp, expflg, nphan[0], nrun[0]);

			if (nexp > 1)
			{
				sscanf(string, "%*s%*d%*d%*d%*f%*d%*d%*d%d%d", &nphan[1],
						&nrun[1]);
				if (nphan[1] <= 0 || nrun[1] <= 0)
				{
					errorc("Invalid number of phantoms and/or runs ",
							"for experiment 2");
				}
			}
		}
		else
		{
			errorc("Error reading supersnark input file at ", string);
			break;
		}

		if (fgets(string, sizeof(string), stdin) == NULL)
		{
			fprintf(stdout, "input error: end of file reached\n");
			break;
		}
		while (isSkip(string))     // skip comment lines and blank lines
		{
			if (fgets(string, sizeof(string), stdin) == NULL)
			{
				fprintf(stdout, "input error: end of file reached\n");
				break;
			}
		}
		if (strncasecmp(string, "dat", 3) == 0)
		{
			sscanf(string, "%*s %s", projflname);
		}
		else
		{
			errorc("Error reading supersnark input file at ", string);
			break;
		}

		if (fgets(string, sizeof(string), stdin) == NULL)
		{
			fprintf(stdout, "input error: end of file reached\n");
			break;
		}
		while (isSkip(string))     // skip comment lines and blank lines
		{
			if (fgets(string, sizeof(string), stdin) == NULL)
			{
				fprintf(stdout, "input error: end of file reached\n");
				break;
			}
		}
		if (strncasecmp(string, "rec", 3) == 0)
		{
			sscanf(string, "%*s %s", recflname);
		}
		else
		{
			errorc("Error reading supersnark input file at ", string);
			break;
		}

		if (fgets(string, sizeof(string), stdin) == NULL)
		{
			fprintf(stdout, "input error: end of file reached\n");
			break;
		}
		while (isSkip(string))     // skip comment lines and blank lines
		{
			if (fgets(string, sizeof(string), stdin) == NULL)
			{
				fprintf(stdout, "input error: end of file reached\n");
				break;
			}
		}
		if (strncasecmp(string, "ana", 3) == 0)
		{
			sscanf(string, "%*s %s", anaflname);
		}
		else
		{
			errorc("Error reading supersnark input file at ", string);
			break;
		}

		supersnk(nexp, expflg, nelem, pixsz, navel, nphan, nrun, ensflname,
				projflname, recflname, anaflname, progName, seed);
	}
	return 0;
}

// this function checks for invalid inputs
void errorchk1(int nexp, int expflg, int nphan1, int nrun1)
{
	if (nexp != 1 && nexp != 2)
	{
		errorc("Invalid number of experiments. ", "See EXPERIMENT command");
	}

	if (expflg != 0 && expflg != 1)
	{
		errorc("Invalid entry for exp-flg. ", "See EXPERIMENT command");
	}

	if (nphan1 <= 0 || nrun1 <= 0)
	{
		errorc("Invalid number of phantoms and/or runs ", "for experiment 1");
	}
}
