/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************


 IMPLEMENTATION OF FOURIER RECONSTRUCTION METHOD
 SEE, E.G., GORDON AND HERMAN, 1974, LUTZ, 1975 AND MERSEREAU,
 1976)
 THE METHOD HAS FOUR STAGES:
 (1) FOURIER TRANSFORMING THE DATA
 (2) FILTERING THE FOURIER TRANSFORM
 (3) INTERPOLATING THE FOURIER TRANSFORM TO A SQUARE GRID
 (4) TAKING THE INVERSE FOURIER TRANSFORM

 TO EXECUTE THE ROUTINE SPECIFY EXEC FOURIER AND CHOOSE PARAMETERS
 AS EXPLAINED BELOW:
 FILTER : BAND  BANDLIMITING FILTER
 SINC  SINC FILTER
 COSI  COSINE FILTER
 HAMM  HAMMING FILTER
 CUTOFF : REAL MODIFIER
 THIS SPECIFIES THE CUTOFF RADIUS AS CUTOFF/(2.0*RINC)
 WHERE RINC IS THE MODIFIED PINC (SEE FRINIT)
 CUTOFF IS SUPPOSED TO BE A NUMBER BETWEEN 0 AND 1
 IF CUTOFF EQ 0  IT IS REASSIGED TO 1.0
 IF CUTOFF LT 0  IT IS REASSIGNED TO 2.0*PRJNUM/(PI*NELEM
 *PIXSIZ)
 IF CUTOFF GT 1 IT IS REASSIGNED TO 1.0
 INTERP : INTEGER MODIFIER
 =1  NEAREST NEIGHBOR IN CARTESIAN SENSE
 =2  NEAREST NEIGHBOR IN RADIAL SENSE
 =3  LINEAR INTERPOLATION
 =4  HARMONIC INTERPOLATION
 SIZE1 : INTEGER MODIFIER
 THE SIZE OF THE ARRAY TO BE USED FOR FOURIER TRANSFORM
 IF OMITTED, IT WILL BE ASSIGNED NRAYS+1
 IF LESS THEN USRAYS+1 IT WILL BE REASSIGNED USRAYS+1
 SIZE2 : INTEGER MODIFIER
 SIZE2*SIZE2 WILL BE THE SIZE OF THE ARRAY TO BE
 BACKTRANSFORMED
 IF OMITTED OR LESS THAN NELEM+1 IT WILL BE REASSIGNED
 NELEM+1

 MORE DETAILED PROCEDURE :
 STEP1 READ 1ST PROJECTION INTO COMMON BLOCK STARTING AT NTEMP+1
 STEP2 REARRANGE THE PROJECTION DATA SO THAT THE ORIGIN IS THE
 FIRST ELEMENT IN THE ARRAY
 PUT INTO COMMON BLOCK STARTING AT NPROJ1+1
 PERFORM FILTERING & FOURIER TRANSFORM, THE TRANSFORM REPLACES THE
 ORIGINAL DATA
 STEP3 REPEAT UNTIL ALL PROJECTIONS ARE USED:
 READ NEXT PROJECTION INTO NTEMP+1
 REARRANGE THE DATA, PUT INTO COMMON BLOCK STARTING AT NPROJ2+1
 OR NPROJ1+1 (LEAST RECENTLY USED)
 PERFORM FILTERING & FOURIER TRANSFORM
 PERFORM INTERPOLATION USING THE DATA IN NPROJ1 AND NPROJ2
 STEP4 READ 1ST PROJECTION, REARRANGE, TRANSFORM, INTERPOLATE BRTWEEN
 1ST & LAST PROJECTIONS
 STEP5 INVERSE TRANSFORM THE INTERPOLATED VALUES
 STEP6 REARRANGE THE INVERSE TRANSFORM AND PUT INTO RECON
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "fourie.h"

#include "foru.h"

BOOLEAN foru_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{
	if (iter > 1)
		return FALSE;

	frinit();
	if (Fourie.iflg == 1)
		return TRUE;

	// Steps 1 - 4
	forwrd();
	if (Fourie.iflg == 1)
		return TRUE;

	// Steps 5, 6
	bckwrd(recon);

	return FALSE;
}

