/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/getnum.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 GETNUM SCANS THE INPUT AREA FOR THE NEXT NUMBER AND RETURNS
 THE INTERNAL FLOATING POINT REPRESENTATION OF THE NUMBER.
 IF THE INPUT AREA DOES NOT CONTAIN A NUMBER, ZERO IS
 RETURNED AND EOL IS SET TO .TRUE.
 */

#include <cstdlib>    
#include <cstdio>
#include <cmath>
#include <cctype>

#include "blkdta.h"
#include "uiod.h"
#include "chtoin.h"

#include "inputfile.h"

REAL InputFile_class::getnum(BOOLEAN NewLine, BOOLEAN* eol)
{
	CHAR nextch;
	REAL mainval = 0, mainsign = 1;
	REAL expval = 0, expsign = 1;
	REAL numdigitsafterdot = 0;

	*eol = FALSE;
	if (NewLine)
		getnxt(FALSE);

	// FIND START OF NUMBER (FIRST DIGIT OR DECIMAL POINT) AND SIGN

	while (true)
	{
		ptr++;
		nextch = data[ptr];
		if (ptr > DATASIZ)     // bug 190 - swr - 12/09/05
		{
			*eol = TRUE;
			return 0.0;
		}
		if (isdigit(nextch))
			break;
		if (nextch == '.')
			break;
		if (nextch == '-')
			mainsign = -1;
		else
			mainsign = 1;
	}

	if (isdigit(nextch))
	{
		while (isdigit(nextch))
		{
			mainval = mainval * 10.0 + chtoin(nextch);
			ptr++;
			nextch = data[ptr];
		}
		if (nextch == '.')
		{
			ptr++;
			nextch = data[ptr];
			while (isdigit(nextch))
			{
				// try to parse (digit*)
				mainval = mainval * 10 + chtoin(nextch);
				numdigitsafterdot += 1;  // number of digits after '.'
				ptr++;
				nextch = data[ptr];
			}
		} // --period read after digit
	} // --digit read before (if at all) period
	else
	{
		if (nextch == '.')
		{
			// parse (.digit+)
			ptr++;
			nextch = data[ptr];
			if (!isdigit(nextch))
			{
				// need a digit there
				fprintf(output,
						"\n **** illegal floating point modifier encountered\n **** program aborted\n");
				exit(-1);
			}
			while (isdigit(nextch))
			{
				mainval = mainval * 10 + chtoin(nextch);
				numdigitsafterdot += 1;  // number of digits after '.'
				ptr++;
				nextch = data[ptr];
			}
		} // --period read before any digit
		else
		{
			// {digit+[.digit*],.digit+} not found
			fprintf(output,
					"\n **** illegal floating point modifier encountered\n **** program aborted\n");
			exit(-1);
		}
	}

	if ((nextch == 'e') || (nextch == 'E'))
	{
		// try to parse ([+,-]digit[digit])
		ptr++;
		nextch = data[ptr];
		if (nextch == '-')
		{
			expsign = -1;
			ptr++;
			nextch = data[ptr];
		}
		else
		{
			if (nextch == '+')
			{
				ptr++;
				nextch = data[ptr];
			}
		}
		if (!isdigit(nextch))
		{
			fprintf(output,
					"\n **** illegal floating point modifier encountered\n **** program aborted\n");
			exit(-1);
		}
		else
		{
			expval = chtoin(nextch);
			ptr++;
			nextch = data[ptr];
		}
		if (isdigit(nextch))
		{
			// second possible digit
			expval = expval * 10 + chtoin(nextch);
			ptr++;
			nextch = data[ptr];
		}
	}

	if (!isspace(nextch))
	{
		// there is junk at the end
		fprintf(output,
				"\n **** illegal floating point modifier encountered\n **** program aborted\n");
		exit(-1);
	}

	return mainsign * mainval * pow(10, (expsign * expval - numdigitsafterdot));
}
