/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/pnchfile.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 */

#include <cstdlib>
#include <cstdio>

#include "blkdta.h"
#include "creacm.h"
#include "geom.h"
#include "spctrm.h"
#include "noise.h"
#include "objects.h"
#include "recfile.h"
#include "pnchfile.h"

PunchFile_class PunchFile;

PunchFile_class::PunchFile_class()
{
	PnchFil = NULL;
}

int PunchFile_class::Open(BOOLEAN append)
{
	const char* mode = (append) ? "a+" : "w";
	if ((PnchFil = fopen("punch", mode)) == NULL)
	{
		return -1;
	}

	return 0;
}

void PunchFile_class::Close()
{
	if (PnchFil != NULL)
	{
		fclose(PnchFil);
		PnchFil = NULL;
	}
}

void PunchFile_class::WriteRecHeader(CHAR* recnam, CHAR* algn, INTEGER count,
		CHAR* prjnam)
{
	fprintf(PnchFil, "%30s alg %4s iter %4i %30s\n", recnam, algn, count,
			prjnam);
	WriteHeader(false);
}

void PunchFile_class::WritePhantomHeader(CHAR* phnnam)
{
	fprintf(PnchFil, "%s\n", phnnam);

	WriteHeader(true);
}

void PunchFile_class::WriteData(REAL* recon, INTEGER nelem)
{
	REAL* jlo = recon;

	for (int n = 0; n < nelem; n++)
	{
		for (int j = 0; j < nelem; j++)
		{
			fprintf(PnchFil, " %10.4f", *jlo);
			if ((j % 7) == 6)
				fprintf(PnchFil, "\n");

			jlo++;
		}
		if (nelem % 7 != 0)
			fprintf(PnchFil, "\n");
	}
}

void PunchFile_class::WriteHeader(BOOLEAN Phantom)
{

	static const CHAR keywdStr[40][5] =
	{ "last", "spec", "mono", "poly", "obje", "phan", "aver", "rays", "geom",
			"para", "dive", "stri", "line", "tang", "arc ", "unif", "vari",
			"angl", "meas", "perf", "nois", "quan", "scat", "mult", "addi",
			"seed", "back", "run ", "modi", "save", "pixe", "equa", "user",
			"and ", "afte", "step", "spac", "pnch", "    ", "    " };

	static const CHAR kindStr[5][5] =
	{ "elip", "rect", "tria", "segm", "sect" };

	static const INTEGER KWI_last = 0;
	static const INTEGER KWI_spec = 1;
	static const INTEGER KWI_mono = 2;
	static const INTEGER KWI_poly = 3;
	static const INTEGER KWI_obj = 4;
	static const INTEGER KWI_pixel = 30;

	// DATA SPACE FOR OBJECTS IN PHANTOM AT TOP OF BLANK COMMON

	BOOLEAN perfct;
	BOOLEAN noisfl;
	BOOLEAN quanfl;

	static const REAL one = 1.0;

	int i, j;

	INTEGER itype;

	quanfl = NoisePar.quanin > 0;
	noisfl = quanfl || NoisePar.ultnfl || NoisePar.addnfl;
	perfct = !(NoisePar.sctnfl || noisfl);

	// ALSO SET VALUES PROPERLY FOR UNAVAILABLE QUANTITIES
	Creacm.nobj = 0;
	Spctrm.nergy = 1;
	Spctrm.energy[0] = -1;

	// WRITE HEADER INFORMATION ON FILE11 FOR PHANTOM & RAYSUMS
	// PUNCH OUT THE HEADER INFORMATION FOR THE PHANTOM AND PICTURES

	// SPECTRUM
	if (Spctrm.nergy == 1)
	{
		fprintf(PnchFil, "%s    %s %4i\n", keywdStr[KWI_spec],
				keywdStr[KWI_mono], Spctrm.energy[0]);
	}

	if (Spctrm.nergy > 1)
	{

		fprintf(PnchFil, "%s    %4s %4i\n", keywdStr[KWI_spec],
				keywdStr[KWI_poly], Spctrm.nergy);

		for (i = 0; i < Spctrm.nergy; i++)
		{
			fprintf(PnchFil, "%5i %5i ", Spctrm.energy[i], Creacm.percnt[i]);
		}
		fprintf(PnchFil, "\n");
	}

	// OBJECTS

	fprintf(PnchFil, "%s\n", keywdStr[KWI_obj]);

	if (Creacm.nobj != 0)
	{

		for (i = 0; i < Creacm.nobj; i++)
		{
			itype = (int) objects[i].type;

			fprintf(PnchFil, "%4s  %9.4f %9.4f %9.4f %9.4f %9.4f %9.4f\n",
					kindStr[itype], objects[i].cx, objects[i].cy, objects[i].u,
					objects[i].v, objects[i].ang, objects[i].den1[0]);

			if (Spctrm.nergy > 1)
			{
				fprintf(PnchFil, "%4i ", i + 1);
				for (j = 1; j < Spctrm.nergy; j++)
				{
					fprintf(PnchFil, " %9.4f", objects[i].den1[j]);
				}
				fprintf(PnchFil, "\n");
			}
		}
	}

	fprintf(PnchFil, "%s %9.4f\n", keywdStr[KWI_last], one);

	// SKIP PHANTOM AVERAGING AND OTHER INFO SUCH AS NELEM FOR RDPROJ

	fprintf(PnchFil, "phan aver 1\n"); // bug 121 - swr - 6/30/05
	int nelem;               // bug 166 - swr - 9/24/05
	RecFile.GetNelem(&nelem);
	REAL pixsiz;
	RecFile.GetPixSize(&pixsiz);
	fprintf(PnchFil, "%s    %i    size    %10.4f\n", keywdStr[KWI_pixel], nelem,
			pixsiz);

	// IF CALLED FROM PUNCH WE ARE DONE
	return;
}

