/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/point.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THIS SUBROUTINE PRODUCES THE POINT BY POINT RESOLUTION OF THE
 RECONSTRUCTED PICTURE.  THIS MEASURE IS SIMILAR TO THAT DEFINED
 IN FRIEDER AND HERMAN (J. THEOR. BIOL., VOL. 33, PP. 189-211),
 HOWEVER, THE NUMBER OF SQUARES OF THE DIFFERENT SIZES HAS BEEN
 GREATLY REDUCED TO MAKE THIS COMPUTATIONALLY FEASIBLE.
 */

#include <cstdio>
#include <cstdlib>
#include <cmath>

#include "blkdta.h"
#include "uiod.h"
#include "eval.h"

void Eval_class::point(REAL* a, REAL* b, REAL* c, INTEGER nelem, INTEGER* nerr,
		REAL*epsil)
{
	int i, j, k;

	INTEGER nh;
	INTEGER nw;
	INTEGER narea;
	REAL cmax;
	INTEGER nnh;
	INTEGER nnw;
	INTEGER ij;
	INTEGER ik;
	INTEGER ii;
	INTEGER ii1;
	INTEGER ii2;
	INTEGER ii3;

	*nerr = (INTEGER) (log((REAL) nelem) / log((REAL) 2) + 1.0);
	if ((*nerr) > 15)
	{
		fprintf(output, "\n **** image is too large for evaluation");
		fprintf(output, "\n **** program aborted\n");
		fprintf(output, "\n");
		exit(-1);
	}

	nh = nelem;
	nw = nelem;
	narea = nelem * nelem;

	// COMPUTE THE DIFFERENCE PICTURE

	for (i = 0; i < narea; i++)
	{
		c[i] = a[i] - b[i];
	}

	for (k = 0; k < *nerr; k++)
	{
		cmax = 0.0;
		// FIND THE MAXIMUM PICTURE ELEMENT
		for (i = 0; i < narea; i++)
		{  //changed from 'i=1' to 'i=0' (Joel)
			cmax = MAX0((REAL) fabs(c[i]), cmax);
		}

		epsil[k] = cmax;

		/*
		 REDUCE THE SIZE OF THE PICTURE BY AVERAGING 2X2 SQUARES,
		 DISCARDING REMAINING ODD SQUARES.  FOR EXAMPLE, IF THE ORIGINAL
		 PICTURE WAS 63X63, SUCESSIVE AVERAGED PICTURE SIZES WOULD BE
		 31X31, 15X15, 7X7, 3X3, AND 1X1.
		 */

		nnh = nh / 2;
		nnw = nw / 2;
		ij = 0;           // used to be 'ij = 1' (Joel 8-12-03)
		ik = 0;           // used to be 'ik = 1' (Joel 8-12-03)
		for (i = 0; i < nnh; i++)
		{

			ii = ik;
			for (j = 0; j < nnw; j++)
			{
				ii1 = ii + nw;
				ii2 = ii + 1;
				ii3 = ii1 + 1;
				c[ij] = (REAL) 0.25 * (c[ii] + c[ii1] + c[ii2] + c[ii3]);
				ii = ii + 2;
				ij = ij + 1;
			}
			ik = ik + nw + nw;
		}

		nh = nnh;
		nw = nnw;
		narea = nh * nw;
	}
	return;
}
