/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_eigval.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

     THIS SUBROUTINE ESTIMATES THE LOWER AND UPPER EIGEN VALUE
     FOR THE MATRIX

           C**P(SA*C*M#*A*M + SB*C*B**L + SC)

     WHERE

         P = T - 1

                          OR FOR THE MATRIX

                     D*M#*A*M*D

     Y IS A WORKING STORAGE ASSUMED TO BE THE RECONSTRUCTION AREA
*/

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"
#include <DIGGauss.h>
#include "blob.h"

#include "quad.h"

void quad_class::eigval(
REAL* y,
BOOLEAN nomin,
REAL* d,
INTEGER* list,
REAL* weight,
REAL tol,
REAL* estmin, //corr. hstau
		REAL* lambda, // corr.hstau
		REAL* s)
{

	REAL max, lower, upper;

	INTEGER i;
	REAL* x;
	REAL presmn;
	INTEGER maxpt;
	REAL xnorml;
	REAL test;

	INTEGER area;
	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	//     ALLOCATE STORAGE AREA

	x = new REAL[area];

//     INITIALIZE X

	for (i = 0; i < area; i++)
	{
		x[i] = 1.0;
	}

	/*
	 FOR METHOD OF ESTIMATING MAX EIGENVALUE SEE
	 A COMPUTER IMPLEMENTATION OF BAYESIAN ANALYSIS
	 OF IMAGE RECONSTRUCTION

	 G. T. HERMAN AND A. LENT

	 INFORMATION AND CONTROL 31,364-384(1976)

	 CALCULATE Y = MATRIX*X
	 */
	for (;;)
	{
		matrix(y, x, d, list, weight, s);

		// SET UPPER AND LOWER BOUNDS
		lower = Consts.infin;
		upper = -Consts.infin;
		*estmin = 0.0;
		// COMPUTE LAMBDA

		*lambda = 0.0;

		for (i = 0; i < area; i++)
		{
			if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
			{
				(*lambda) += y[i];

				// COMPUTE UPPER AND LOWER BOUND CHECK  FOR POSSIBLE DIVISION BY ZERO

				if (x[i] <= Consts.zero)
				{
					fprintf(output, "\n\n      ***warning***");
					fprintf(output, "\n      z(%5i) = %15.6e", i, x[i]);
					fprintf(output,
							"\n      lower and upper bound are ignored");
				}

				else
				{
					test = y[i] / x[i];

					// CHECK FOR NONPOSITIVE RESULTS

					if (test <= 0.0)
					{
						fprintf(output, "\n\n      ***warning***");
						fprintf(output,
								"\n      negative value in matrix*x(%5i) = %e",
								i, y[i]);
						fprintf(output, "\n      valve ignored");
					}
					else
					{
						if (test < lower)
							lower = test;
						if (test > upper)
							upper = test;
					}
				}
			}
		}

		if (Blob.pix_basis)
		{
			(*lambda) /= ((REAL) (area));
		}
		else
		{
			(*lambda) /= ((REAL) (area / 2.0));
		}

		if (popt >= 1)
		{
			fprintf(output,
					"\n\n      lower estimate for largest eigenvalue %15.6e",
					lower);
			fprintf(output,
					"\n            estimate for largest eigenvalue %15.6e",
					*lambda);
			fprintf(output,
					"\n      upper estimate for largest eigenvalue %15.6e",
					upper);        //bug 147, wei, 2005/10
		}

		// TEST FOR TERMINATION

		if (((upper - lower) / upper) < tol)
		{
			break;
		}

		// EVALUATE NEXT X

		for (i = 0; i < area; i++)
		{
			if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
			{
				x[i] = y[i] / (*lambda);
			}
		}
	}

	*estmin = upper;

	if (!nomin)
	{

		/*
		 ESTIMATE SMALLEST EIGEN VALUE
		 FOR METHOD SEE ABOVE REFERENCE AND

		 A FIRST COURSE IN NUMERICAL ANALYSIS

		 A. RALSTON

		 MCGRAW-HILL 1965 PAGE 475

		 INITIALIZE X
		 */

		for (i = 0; i < area; i++)
		{

			if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
			{
				x[i] = Gauss(0.0, 1.0);
			}
		}

		// CALCULATE Y = UPPER*X - MATRIX*X

		for (;;)
		{
			max = -Consts.infin;
			matrix(y, x, d, list, weight, s);
			for (i = 0; i < area; i++)
			{
				if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
				{
					y[i] = upper * x[i] - y[i];
				}
			}

			presmn = *estmin;

			if (!((t == 0) || (copt != 4)))
			{

				// NON SYMETRIC MATRIX

				// FIND MAX OF x[i]

				for (i = 0; i < area; i++)
				{
					if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
					{
						if (x[i] > max)
						{
							max = x[i];
							maxpt = i;
						}
					}
				}

				// NEW ESTIMATES FOR MINIMUM EIGENVALUE

				*estmin = y[maxpt] / max;
			}
			else
			{
				//    SYMETRIC MATRIX

				max = 0;
				*estmin = 0.0;
				for (i = 0; i < area; i++)
				{
					if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
					{
						max += x[i] * x[i];
						(*estmin) += y[i] * x[i];
					}
				}
				(*estmin) /= max;
			}

			if (popt >= 1)
			{
				fprintf(output, "\n\n      estimate for largest eigenvalue of upper*i-matrix %15.6e",
						*estmin); //bug 147, wei, 2005/10
			}

			xnorml = *estmin;
			if (fabs(*estmin) <= Consts.zero)
				xnorml = 1.0;
			if (fabs(*estmin - presmn) / xnorml <= tol)
			{
				break;
			}

			for (i = 0; i < area; i++)
			{
				if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
				{
					x[i] = y[i] / xnorml;
				}
			}
		}
	}
	*estmin = upper - (*estmin);

	if (popt >= 1)
	{
		fprintf(output, "\n\n      estimate for smallest eigenvalue %15.6e", *estmin);  //bug 147, wei, 2005/10
	}

	delete[] x;  // bug 92 - Lajos - 03/02/2005

	return;
}
