/*
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/superior.cpp $
$LastChangedRevision: 158 $
$Date: 2014-07-30 16:59:01 -0400 (Wed, 30 Jul 2014) $
$Author: olangthaler $
*/

#include <cstdlib>
#include <iostream>
#include <cmath>
#include <math.h>
#include <stdlib.h>
#include "superior.h"
#include "nav.h"
#include "nav1.h"
#include "nav2.h"
#include "nav3.h"
#include "nav4.h"
#include "nav5.h"
#include "sec_cri.h"
#include "sec_cri1.h"
#include "sec_cri2.h"
#include "sec_cri3.h"
#include "sec_cri4.h"
#include "sec_cri5.h"
#include "uiod.h"
#include "int2str.h"
#include "infile.h"
#include "geom.h"
#include "projfile.h"
#include "blob.h"
#include "DIGRand.h"

using namespace std;

sec_cri1_class sc1;
sec_cri2_class sc2;
sec_cri3_class sc3;
sec_cri4_class sc4;
sec_cri5_class sc5;

sec_cri_class* scClasses[] =
{
	&sc1,
	&sc2,
	&sc3,
	&sc4,
	&sc5
};

nav1_class nav1;
nav2_class nav2;
nav3_class nav3;
nav4_class nav4;
nav5_class nav5;

nav_class* navClasses[] =
{
	&nav1,
	&nav2,
	&nav3,
	&nav4,
	&nav5
};

class superior_class SuperSet =
{
	FALSE,
	0,
	0,
	0,
	NULL,
	NULL,
	FALSE,
	0
};

void initSuperiorization()
{
	INTEGER word;
	REAL number;
	BOOLEAN eol;

	// enable superiorization
	SuperSet.superiorizationEnabled = TRUE;
	cout << "\n         Superiorization is enabled";

	// read N
	word = InFile.getint(FALSE, &eol);

	if (word <= 0)
	{
		cout << "\n **** N must be greater than zero";
		cout << "\n **** program aborted\n";
		exit(-1);
	}
	SuperSet.N = word;
	cout << "\n         N = " << SuperSet.N;

	// read a
	number = InFile.getnum(FALSE, &eol);

	if (number <= 0)
	{
		cout << "\n **** a must be greater than zero";
		cout << "\n **** program aborted\n";
		exit(-1);
	}
	if (number >= 1)
	{
		cout << "\n **** a must be less than one";
		cout << "\n **** program aborted\n";
		exit(-1);
	}
	SuperSet.a = number;
	cout << "\n         a = " << SuperSet.a;

	// read b
	number = InFile.getnum(FALSE, &eol);

	if (number <= 0)
	{
		cout << "\n **** b must be greater than zero";
		cout << "\n **** program aborted\n";
		exit(-1);
	}
	SuperSet.b = number;
	cout << "\n         b = " << SuperSet.b;

	// read secondary criterion
	static const INTEGER criterion[] =
	{
		CHAR2INT('t', 'v', 'a', 'r'),
		CHAR2INT('s', 'm', 'o', 'o'),
		CHAR2INT('s', 'c', 'r', '3'),
		CHAR2INT('s', 'c', 'r', '4'),
		CHAR2INT('s', 'c', 'r', '5')
	};

	word = InFile.getwrd(FALSE, &eol, criterion, 5);

	for (INTEGER i = 0; i < 5; i++)
	{
		if (word == criterion[i])
		{
			cout << "\n         secondary criterion: " << int2str(criterion[i]);

			SuperSet.secondaryCriterion = scClasses[i];
			SuperSet.secondaryCriterion->Init();

			// set the default non-ascending vector for the selected secondary criterion
			SuperSet.nonascendingVector = navClasses[i];
			SuperSet.nonascendingVector->Init();

			break;
		}
	}

	if (SuperSet.secondaryCriterion == NULL)
	{
		cout << "\n **** unknown secondary criterion ", int2str(word);
		cout << "\n **** program aborted\n";
		exit(-1);
	}

	// set default values of the optional commands
	SuperSet.report = false;
	SuperSet.alternativel=0;
	SuperSet.positivity = false;

	// read remaining optional commands: posi, altl (alternative l) or rprt
	static const INTEGER optionsf[] =
	{
		CHAR2INT('a', 't', 'l', '1'),
		CHAR2INT('a', 't', 'l', '2'),
		CHAR2INT('r', 'p', 'r', 't'),
		CHAR2INT('p', 'o', 's', 'i')
	};

	word = InFile.getwrd(FALSE, &eol, optionsf, 4);

	for (INTEGER i = 0; i < 4; i++)
	{
		if (word == optionsf[i])
		{
			if (i==0)
			{
				SuperSet.alternativel=1;
				cout << "\n         alternative l handling 1 is enabled";
			}
			else if (i==1)
			{
				SuperSet.alternativel=2;
				cout << "\n         alternative l handling 2 is enabled";
			}
			else if (i==2)
			{
				SuperSet.report = true;
				cout << "\n         reporting is enabled";
				cout << "\n         reporting file: RPRTsuperiorization";

				SuperSet.skips = InFile.getnum(FALSE, &eol);
				if (SuperSet.skips>0)
				{
					cout << "\n         report skip factor " << SuperSet.skips;
				}
				else
				{
					cout << "\n         reporting on every iteration";
				}
			}
			else if (i==3)
			{
				SuperSet.positivity=true;
				cout << "\n         positivity constraint is enabled";
			}
		}
	}

	// read remaining optional commands: altl (alternative l) or rprt
	static const INTEGER options[] =
	{
		CHAR2INT('a', 't', 'l', '1'),
		CHAR2INT('a', 't', 'l', '2'),
		CHAR2INT('r', 'p', 'r', 't')
	};

	word = InFile.getwrd(FALSE, &eol, options, 3);

	for (INTEGER i = 0; i < 3; i++)
	{
		if (word == options[i])
		{
			if (i==0)
			{
				SuperSet.alternativel=1;
				cout << "\n         alternative l handling 1 is enabled";
			}
			else if (i==1)
			{
				SuperSet.alternativel=2;
				cout << "\n         alternative l handling 2 is enabled";
			}
			else if (i==2)
			{
				SuperSet.report = true;
				cout << "\n         reporting is enabled";
				cout << "\n         reporting file: RPRTsuperiorization";

				SuperSet.skips = InFile.getnum(FALSE, &eol);
				if (SuperSet.skips>0)
				{
					cout << "\n         report skip factor " << SuperSet.skips;
				}
				else
				{
					cout << "\n         reporting on every iteration";
				}
			}
		}
	}

	// read last possible command (rprt)
	INTEGER rprt = CHAR2INT('r','p','r','t');
	if (InFile.getwrd(FALSE, &eol, &rprt, 1) == rprt)
	{
		SuperSet.report = true;
		cout << "\n         reporting is enabled";
		cout << "\n         reporting file: RPRTsuperiorization";

		SuperSet.skips = InFile.getnum(FALSE, &eol);
		if (SuperSet.skips>0)
		{
			cout << "\n         report skip factor " << SuperSet.skips;
		}
		else
		{
			cout << "\n         reporting on every iteration";
		}
	}

	// "set l = -1"
	SuperSet.l = -1;
}

BOOLEAN executeSuperiorization(REAL* recon, INTEGER* list, REAL* weight, INTEGER count, alg_class* algorithm)
{
	double min = pow(10,-15);
	int pixelcount = 0;

	if(Blob.pix_basis)
	{
		pixelcount = GeoPar.area;
	}
	else
	{
		cout << "\nError: Superiorization does currently not support blobs, skipping Superiorization.";

		// execute one iteration of the selected algorithm
		return algorithm->Run(recon, list, weight, count);
	}

	// user defined superiorization parameters
	INTEGER N = SuperSet.N;
	REAL a = SuperSet.a;
	REAL b = SuperSet.b;
	sec_cri_class* secCrit = SuperSet.secondaryCriterion;
	nav_class* naV = SuperSet.nonascendingVector;

	// xkn, vkn and z are declared here so that memory is only allocated once
	REAL* xkn = new REAL[pixelcount];
	REAL* vkn = new REAL[pixelcount];
	REAL* z = new REAL[pixelcount];

	// things that should only be done on the first iteration
	// "set k = 0"
	// taken care of by loop in exalg
	// "set xk = xdash"
	// not necessary (accessing recon directly)
	// "set l = -1"
	INTEGER l;
	if (SuperSet.alternativel==1)
	{
		l = count-2;
	}
	else if (SuperSet.alternativel==2 && count>1)
	{
		l = (int)round(Rand()*(SuperSet.l-count)) + count - 1;
	}
	else if (count==1)
	{
		SuperSet.l = l = -1;
	}
	else
	{
		l = SuperSet.l;
	}

	// "repeat"
	// taken care of by loop in exalg
	// "set n = 0"
	INTEGER n = 0;
	// "set xkn = xk"
	memcpy(xkn, recon, pixelcount * sizeof(REAL));
	// "while n < N"
	REAL phixk=secCrit->Run(recon); // only needs to be processed once
	while (n < N)
	{
		// "set vkn to be a nonascending vector for phi at xkn"
		naV->Run(xkn, vkn);

		// "set loop = true"
		// "while (loop)"

		while (true)
		{
			// "set l = l+1"
			l++;
			// "set betakn = mul"
			REAL betakn = b*pow(a, l);
			if (betakn<min) betakn=0;
			// "set z = xkn + betakn*vkn"
			for (INTEGER i = 0; i < pixelcount; i++) z[i] = xkn[i] + (betakn * vkn[i]);
			// "if z e delta and phi(z) <= phi(xk) then"
			BOOLEAN inDelta = true;
			if (SuperSet.positivity)
			{
				for (INTEGER i = 0; i < pixelcount; i++) if (z[i] < 0) inDelta=false;
			}
			if (inDelta && secCrit->Run(z) <= phixk)
			{
				// "set n = n+1"
				n++;
				// "set xkn = z"
				memcpy(xkn, z, pixelcount * sizeof(REAL));
				// "set loop = false"
				break;
			}
		}
	}
	// "set xk+1 = B*xkN"
	memcpy(recon, xkn, pixelcount * sizeof(REAL));

	delete[] (xkn);
	delete[] (vkn);
	delete[] (z);

	// save value of l for next iteration (checking if this is necessary will take as long as saving it)
	SuperSet.l = l;

	// reporting output
	if (SuperSet.report && (count==1 || SuperSet.skips==0 || (count%(INTEGER)SuperSet.skips)==0))
	{
		REAL preAlgo = secCrit->Run(recon);
		BOOLEAN error = algorithm->Run(recon, list, weight, count);
		cout << "\n              value of l: " << l;
		cout << "\n              value of phi before algorithm operator: " << preAlgo;
		REAL postAlgo = secCrit->Run(recon);
		cout << "\n              value of phi after algorithm operator:  " << postAlgo;

		FILE* RPRTfile;
		if (count==1)
		{
			RPRTfile = fopen("RPRTsuperiorization", "w");
			fprintf(RPRTfile,"superiorization reporting output");
			fprintf(RPRTfile,"\n    iter         l     phi pre-algorithm      phi post-algorithm");
			fprintf(RPRTfile,"\n %5i  %10i  %20.9f  %20.9f", count, l, preAlgo, postAlgo);
		}
		else
		{
			RPRTfile = fopen("RPRTsuperiorization", "a");
			fprintf(RPRTfile,"\n %5i  %10i  %20.9f  %20.9f", count, l, preAlgo, postAlgo);
		}
		fclose(RPRTfile);

		return error;
	}

	// execute one iteration of the selected algorithm
	return algorithm->Run(recon, list, weight, count);

	// "set k = k+1"
	// taken care of by loop in exalg
}
