/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/trm6.cpp $
 $LastChangedRevision: 145 $
 $Date: 2014-07-17 19:49:25 -0400 (Thu, 17 Jul 2014) $
 $Author: olangthaler $
 ***********************************************************

 Residual based stopping criterion

 */

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <math.h>
#include "trm6.h"
#include "anglst.h"
#include "projfile.h"
#include "pseudo.h"
#include "consts.h"
#include "infile.h"
#include "blob.h"

using namespace std;

void trm6_class::Init()
{
	// read desired epsilon from command
	BOOLEAN eol;
	epsilon = InFile.getnum(FALSE, &eol);

	if (epsilon < Consts_class::zero)
	{
		cout << "\n **** epsilon must be specified and >= ZERO";
		cout << "\n **** program aborted\n";
		exit(-1);
	}

	// read reporting command (disabled by default)
	INTEGER rprt = CHAR2INT('r', 'p', 'r', 't');
	if (InFile.getwrd(FALSE, &eol, &rprt, 1) == rprt)
	{
		report = true;
		cout << "\n         reporting is enabled";
		cout << "\n         reporting file: RPRTresi";

		skips = InFile.getnum(FALSE, &eol);
		if (skips > 0)
		{
			cout << "\n         report skip factor " << skips;
		}
		else
		{
			cout << "\n         reporting on every iteration";
		}
	}
	else
	{
		report = false;
		cout << "\n         reporting is disabled";
	}

	cout << "\n         epsilon = " << epsilon;
}

BOOLEAN trm6_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{
	REAL* reconp = recon;
	// if currently blob, pixelize it
	if (!Blob.pix_basis)
	{
		reconp = new REAL[GeoPar.area];
		Blob.blob2pix(recon, reconp);
	}

	// variables needed during computation of the current residual
	INTEGER numb = 0;
	REAL snorm = 0;
	REAL raysum = 0;
	REAL psum = 0;
	REAL rdis = 0;
	REAL theta = 0;
	REAL sinth = 0;
	REAL costh = 0;
	BOOLEAN linef = true;

	// compute current residual
	for (int i = 0; i < GeoPar.prjnum; i++)
	{
		REAL rinc = GeoPar.pinc;
		Anglst.getang(i, &theta, &sinth, &costh);

		if (GeoPar.vri)
		{
			rinc = GeoPar.pinc * MAX0(fabs(sinth), fabs(costh));
		}

		for (int j = GeoPar.fusray; j <= GeoPar.lusray; j++)
		{
			raysum = Anglst.prdta(i, j);
			psum = pseudo(reconp, i, j, list, weight, &numb, &snorm, linef,
					TRUE);

			if (numb != 0)
			{
				if (linef && !GeoPar.line)
					raysum /= rinc;
				if (!linef && GeoPar.line)
					raysum *= rinc;
				rdis += pow((psum - raysum), 2);
			}
		}
	}
	REAL currentEpsilon = sqrt(rdis);
	if (!Blob.pix_basis)
	{
		delete[] reconp;
	}

	if (currentEpsilon <= epsilon)
	{
		// additional output
		if (report)
		{
			cout << "\n         current epsilon (residual) = " << currentEpsilon;

			FILE* RPRTfile;
			if (iter==1)
			{
				RPRTfile = fopen("RPRTresi", "w");
				fprintf(RPRTfile,"Residual based stopping criterion reporting output");
				fprintf(RPRTfile,"\n    iter    Residual");
				fprintf(RPRTfile,"\n %5i  %20.9f", iter, currentEpsilon);
			}
			else
			{
				RPRTfile = fopen("RPRTresi", "a");
				fprintf(RPRTfile,"\n %5i  %20.9f", iter, currentEpsilon);
			}
			fclose(RPRTfile);
		}

		return true;
	}
	else
	{
		// additional output
		if (report && (iter == 1 || skips == 0 || (iter % (INTEGER) skips) == 0))
		{
			cout << "\n         current epsilon (residual) = " << currentEpsilon;

			FILE* RPRTfile;
			if (iter==1)
			{
				RPRTfile = fopen("RPRTresi", "w");
				fprintf(RPRTfile,"Residual based stopping criterion reporting output");
				fprintf(RPRTfile,"\n    iter      Residual");
				fprintf(RPRTfile,"\n %5i  %20.9f", iter, currentEpsilon);
			}
			else
			{
				RPRTfile = fopen("RPRTresi", "a");
				fprintf(RPRTfile,"\n %5i  %20.9f", iter, currentEpsilon);
			}
			fclose(RPRTfile);
		}

		return false;
	}
}
