/** @file proj.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "variables.hpp"
#include "proj.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <qmessagebox.h>

Snarkproj::Snarkproj()
{
  proj=NULL;
  max=-100000;
  min=100000;
  isloaded=false;
  fileformat=DIGDataFormat_ASCII;
}

Snarkproj::Snarkproj(DIGDataFormat format)
{
  unsigned int i,j;
  double *tmpimg;

  max=-100000;
  min=100000;
  tmpimg=(double *)malloc(sizeof(double)*usrays);
  proj=matrix(0,prjnum,0,usrays);
  if(!format) { /* format=0 (ASCII)  */
    projformat=DIGDataFormat_ASCII;
    for(i=0;i<prjnum;i++) {
//       digprojfil.SelectArraySet(i);
      digprojfil.SelectProj(i);
//       digprojfil.SelectArray(0);
//       digprojfil.GetArrayData(tmpimg);
      digprojfil.GetProjData(tmpimg);
      for(j=0;j<usrays;j++) {
	proj[i][j]=tmpimg[j];
        if(proj[i][j]>max)
	  max=proj[i][j];
        if(proj[i][j]<min)
	  min=proj[i][j];
      }
    }
  }
  else { /* format=1 (BINARY) */
    projformat=DIGDataFormat_BINARY;
    for(i=0;i<prjnum;i++) {
      //      digprojfil.SelectArraySet(i);
      digprojfil.SelectProj(i);
      //      digprojfil.SelectArray(0);
      //      digprojfil.GetArrayData(tmpimg);
      digprojfil.GetProjData(tmpimg);
      for(j=0;j<usrays;j++) {
	proj[i][j]=tmpimg[j];
        if(proj[i][j]>max)
	  max=proj[i][j];
        if(proj[i][j]<min)
	  min=proj[i][j];
      }
    }
  }
 isloaded=true;
 free(tmpimg);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Snarkproj::~Snarkproj()
{
    // no need to delete child widgets, Qt does it all for us
}

// void Snarkproj::readProj(DIGDataFormat format)
void Snarkproj::readProj()
{
  int i,j;
  double *tmpimg;

  max=-100000;
  min=100000;
  tmpimg=(double *)malloc(sizeof(double)*usrays);
  proj=matrix(0,prjnum,0,usrays);
//   if(!format) { /* format=0 (DIGDataFormat_ASCII)  */
//     for(i=0;i<prjnum;i++) {
//       //      digprojfil.SelectArraySet(i);
//       digprojfil.SelectProj(i);
//       //      digprojfil.SelectArray(0);
//       //      digprojfil.GetArrayData(tmpimg);
//       digprojfil.GetProjData(tmpimg);
//       for(j=0;j<usrays;j++) {
// 	proj[i][j]=tmpimg[j];
//         if(proj[i][j]>max)
// 	  max=proj[i][j];
//         if(proj[i][j]<min)
// 	  min=proj[i][j];
//       }
//     }
//   }
//  else { /* format=1 (DIGDataFormat_BINARY) */
    for(i=0; i< (static_cast<int>(prjnum)) ; i++) {
      //      digprojfil.SelectArraySet(i);
      digprojfil.SelectProj(i);
      //      digprojfil.SelectArray(0);
      //      digprojfil.GetArrayData(tmpimg);
      digprojfil.GetProjData(tmpimg);
      for(j=0; j< (static_cast<int>(usrays)) ; j++) {
	proj[i][j]=tmpimg[j];
        if(proj[i][j]>max)
	  max=proj[i][j];
        if(proj[i][j]<min)
	  min=proj[i][j];
      }
    }
    //  }
 isloaded=true;
 free(tmpimg);
}

double** Snarkproj::getProj()
{
  return proj;
}

double Snarkproj::getMax()
{
  return max;
}

double Snarkproj::getMin()
{
  return min;
}

bool Snarkproj::isLoaded()
{
  return isloaded;
}

double** Snarkproj::matrix(int nrl,int nrh,int ncl,int nch)
/*  Allocates a double array with range [npl .. nph][nrl .. nrh][ncl .. nch]  */
{
   int j;
   double **m;

   m = (double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
   if (!m) {
       printf("Allocation failure in matrix() (y)\n");
       exit(-1);
   }
   m -= nrl;
   for(j=nrl; j<=nrh; j++) {
     m[j] = (double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
     if(!m[j]) {
       printf("Allocation failure in matrix() (x)\n");
       exit(-1);
     }
     m[j] -= ncl;
   }
   return m;
}

