/** @file experiment.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "experiment.h"

#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/* 
 *  Constructs a experimentwindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho
@version 1.0 */
experimentwindow::experimentwindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "experimentwindow" );
    resize( 400, 150 ); 
    setMinimumSize( QSize( 400, 150 ) );
    setMaximumSize( QSize( 400, 150 ) );
    setCaption( tr( "Experiment's Comments Window" ) );

    TextLabel1 = new QLabel( this, "TextLabel1" );
    TextLabel1->setGeometry( QRect( 15, 30, 15, 30 ) ); 
    QFont TextLabel1_font(  TextLabel1->font() );
    TextLabel1_font.setFamily( "adobe-helvetica" );
    TextLabel1_font.setPointSize( 16 );
    TextLabel1->setFont( TextLabel1_font ); 
    TextLabel1->setText( tr( "*" ) );

    comments = new QLineEdit( this, "comments" );
    comments->setGeometry( QRect( 35, 25, 335, 30 ) ); 
    comments->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, comments->sizePolicy().hasHeightForWidth() ) );

    Line9 = new QFrame( this, "Line9" );
    Line9->setGeometry( QRect( -5, 75, 420, 16 ) ); 
    Line9->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 50, 90, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(accept()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 250, 90, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/*  
 *  Destroys the object and frees any allocated resources
 */
experimentwindow::~experimentwindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/*  
 *  Main event handler. Reimplemented to handle application
 *  font changes
 */
bool experimentwindow::event( QEvent* ev )
{
    bool ret = QDialog::event( ev ); 
    if ( ev->type() == QEvent::ApplicationFontChange ) {
	QFont TextLabel1_font(  TextLabel1->font() );
	TextLabel1_font.setFamily( "adobe-helvetica" );
	TextLabel1_font.setPointSize( 16 );
	TextLabel1->setFont( TextLabel1_font ); 
    }
    return ret;
}

/** Returns a string of comments 
@param void
@return mode
@author Bruno M. Carvalho
@version 1.0 */
QString experimentwindow::getOutput()
{
  QString output;

  output.sprintf("* ");
  output+=comments->text();
  return output;
}
