/** @file misc.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "variables.h"
#include "misc.h"
#include <string.h>
#include <cstdio>
#include <cmath>

int toint(const char c);

// Deniz thought this was overly buggy
/** Extracts a float from a string
@param void
@author Bruno M. Carvalho
@version 1.0 */
// float strtof(const char *str) {
//   printf("string `%s' passed to strtof() defined in misc.cpp\n", str);
//   float result=0,fraction=0,s=1,sexp=1;
//   int i=0,nf=0,exp=0;

//   result=0;
//   errnum=0;
//   if(str[i]=='\0') {
//     errnum=2;
//     return result;
//   }
//   while(str[i]!='-' && !isdigit(str[i])) {
//     i++;
//   }
//   if(str[i]=='-') {
//     s=-1;
//     i++;
//   }
//   while(isdigit(str[i])) {
//     result=result*10+toint(str[i]);
//     i++;
//   }
//   if(str[i]=='.') {
//     i++;
//   }
//   if( (str[i]=='e') || (str[i]=='E') )  {
//     i++;
//     exp=1;
//     if(str[i]=='-') {
//       sexp=-1;
//       i++;
//     }
//   }
//   while(isdigit(str[i])) {
//     fraction=fraction*10+toint(str[i]);
//     i++;
//     nf++;
//   }
//   while(str[i]) {
//     if(str[i]!=32) errnum=1;
//     i++;
//   }
//   if(nf>0)
//     if(exp)
//       result=s*result*pow(10,sexp*fraction);
//     else 
//       result=s*(result+fraction/(10*nf));
//   else
//     result=s*result;
//   return result;
// }


/* @author deniz
   @version 3.0
   will implement:
   [-]{digit+[.digit*],.digit+}[{E,e}[+,-]digit[digit]]
   as decided on 2005.08.11 in group meeting
*/
float strtof(const char *str) {
  //  printf("string `%s' passed to strtof() defined in misc.cpp\n", str);
  // new parse 'philosophy': get the number as integer
  // if there is stuff after the period, count them
  int mainval=0, mainsign=1;
  int expval=0, expsign=1;
  int pos=0, numdigitsafterdot=0;
  errnum=0;
  while(isspace(str[pos])) {
    pos++;
  }
  if(str[pos]=='-') {  //  [-]
    mainsign= -1;
    pos++;
  }
  // adjust at the end
  // now get:  {digit+[.digit*],.digit+}
  if(isdigit(str[pos])) { // try to parse (digit+[.digit*])
    while(isdigit(str[pos])) {
      mainval = mainval*10 + toint(str[pos]);
      pos++;
    }
    if(str[pos]=='.') {
      pos++;
      while(isdigit(str[pos])) { // try to parse (digit*)	
	mainval = mainval*10 + toint(str[pos]);
	numdigitsafterdot++;  // number of digits after '.'
	pos++;
      }
    } // --period read after digit
  } // --digit read before (if at all) period
  else if(str[pos]=='.') { // parse (.digit+)
    pos++;
    if(!isdigit(str[pos])) {
      errnum = 2; // need a digit there
      return 0.0;
    }
    while(isdigit(str[pos])) {
      mainval = mainval*10 + toint(str[pos]);
      numdigitsafterdot++;  // number of digits after '.'
      pos++;
    }
  } // --period read before any digit
  else {   // {digit+[.digit*],.digit+} not found
    errnum = 1;
    return 0.0;
  }
  if( (str[pos]=='e') || (str[pos]=='E') )  { // try to parse ([+,-]digit[digit])
    pos++;
    if(str[pos]=='-') {
      expsign= -1;
      pos++;
    } else if(str[pos]=='+') { 
      pos++; 
    }
    if(!isdigit(str[pos])) { // try to parse (digit[digit])
      errnum = 4; // no exp digit
      return 0.0;
    } else {
      expval = toint(str[pos]);
      pos++;
    }
    if(isdigit(str[pos])) { // second possible digit
      expval = expval*10 + toint(str[pos]);
      pos++;
    }
  }
  while(isspace(str[pos])) {
    pos++;
  }
  if('\0' != str[pos]) {  // there is junk
    errnum = 5;
    return 0.0;
  }
  float retval = mainsign*mainval * pow(10, (expsign*expval - numdigitsafterdot));
  //  printf("%8.8f\n", retval);
  return retval; // (mainsign*mainval * pow(10, (expsign*expval - numdigitsafterdot) ) );
} // strtof()


/** Extracts an int from a string
@param void
@author Bruno M. Carvalho
@version 1.0 */
int strtoi(const char *str) {
  int result=0,i=0,t;

  result=0;
  errnum=0;
  if(str[i]=='\0') {
    errnum=2;
    return result;
  }
  t=strcmp(str,"");
  if(t) {
    while(!isdigit(str[i])) {
      i++;
    }
    while(isdigit(str[i])) {
      result=result*10+toint(str[i]);
      i++;
    }
    while(str[i]) {
      if(str[i]!=32) errnum=1;
      i++;
    }
  }
  return result;
}

/** checks if there are leading spaces
@param void
@author Bruno M. Carvalho
@version 1.0 */
int isintspace(const char *str) {
  int i=0;

  errnum=0;
  if(!strcmp(str,""))
    return 0;
  while(isdigit(str[i]) || str[i]==' ') {
    i++;
  }
  if(str[i]) return 0;
  else  return 1;
}

/** Checks if line is a snark14 formatline
@param void
@author Bruno M. Carvalho
@version 1.0 */
int isformatline(const char *str) {
  int i=0;

  errnum=0;
  while(str[i]!='(')  i++;
  i++;
  while(isdigit(str[i]))  i++;
  if(stolower(str[i])!='f')
    errnum=1;
  if(errnum) return 0;
  else  {
    i++;
    while(isdigit(str[i]))  i++;
    if(str[i]!='.') {
      errnum=1;
      return 0;
    }
    i++;
    while(isdigit(str[i]))  i++;
    if(str[i]!=')') {
      errnum=1;
      return 0;
    }
    i++;
  if(str[i]) return 0;
  else  return 1;
  }
}

/** Converts a char into an int 
@param void
@author Bruno M. Carvalho
@version 1.0 */
int toint(const char c) {

  if(c>='0' && c<= '9') return c - '0';
  else return 0;
}

/** converts a char into lower case
@param void
@author Bruno M. Carvalho
@version 1.0 */
int stolower(char c) {

  if(isupper(c)) return tolower(c);
  else return c;
}
